<?php
/**
 * Plugin Name: Emergency Management
 * Description: Provides means to reset all or selectable passwords (roles, users) and/or to delete (the related) sessions, to define a role-based password expiry, to renew your security KEYs & SALTs and to define & monitor your required password complexity/strength
 *
 * Inspired by: Joe Sexton (PW-Check with Regular Expressions and related basic functions structure)
 * Inspired by: andymoyle (Plugin "Emergency password reset V6.1" providing this as a kind of 'brute force' functionality)
 * Inspired by: Miller Media (Plugin "Expire User Passwords", providing some basic ideas of functionality but in conflict with the plugin 'Theme My Login')
 *
 * Author: Michael Finkenberger
 * Plugin URI: https://de.wordpress.org/plugins/emergency-management/
 *
 * Text Domain: emergency-management
 * Domain Path: /languages
 *
 * Requires at least: 4.8
 * Tested up to: 6.0.1
 * Requires PHP: 5.6
 *
 * Licence: GPLv2 or later
 *
 * Version: 1.4.0.0
 * Date: 18.05.2021
 */

if(!defined('ABSPATH')) die();// no direct access

const MFEM_VERSION = '1.4.0.0'; // The current plugin version
global $thanks_for_using_emergency_management; // translated (WordPress action hook 'init')
global $mfem_session_tooltip; // translated (WordPress action hook 'init')
global $you_cannot_reuse_your_old_password; // translated (WordPress action hook 'init')



//delete_option('mfem_activated'); // just for switching on and off during testing
//delete_option('mfem_admin_notice'); // just for switching on and off during testing
//delete_option('mfem_email_content'); // just for switching on and off during testing
//delete_option('mfem_pw_strength'); // just for switching on and off during testing
//delete_option('mfem_pw_expiry'); // just for switching on and off during testing



// Provide functionality upon plugin activation
function mfem_emergency_management_activate() {
  
// Do some initial stuff...
  delete_option('mfem_version'); // no longer used from 1.4.0.0
  if ( !get_option('mfem_activated') )
    add_option('mfem_activated', 'emergency-management-yes', '', 'yes'); // I am here - provided for future add-ons
  else update_option('mfem_activated', 'emergency-management-yes', 'yes'); // I am back
  
// Add a password renewal timestamp to each user's meta data as if the password was just renewed - for later use = the starting point
  $users = get_users( 'orderby=meta_value&meta_key=last_name' );
  foreach ( $users as $user ) {
    $mfem_user_id = $user->ID;
    if ( ! metadata_exists( 'user', $mfem_user_id, 'mfem_user_pw_reset' ) ) // Add only, if it doesn't exist, yet (...well, it shouldn't upon activation!)
      add_user_meta( $mfem_user_id, 'mfem_user_pw_reset', wp_date('U'), true ); // Unix timestamp (seconds since Unix Epoch)
  }
}
register_activation_hook( __FILE__, 'mfem_emergency_management_activate' );



// Do some stuff upon plugin deactivation
function mfem_emergency_management_deactivate() {
  
// Update the show towards being still present but deactivated now
  update_option('mfem_activated', 'emergency-management-no','yes'); // I am here but deactivated

}
register_deactivation_hook( __FILE__, 'mfem_emergency_management_deactivate' );



// Clean up upon plugin uninstall
function mfem_emergency_management_uninstall() {
  
// Delete all options used in the emergency-management plugin
  delete_option('mfem_activated'); // good-bye
  delete_option('mfem_admin_notice'); // just in case as it always has a short life here...
  delete_option('mfem_email_content');
  delete_option('mfem_pw_strength');
  delete_option('mfem_pw_expiry');
  delete_option('mfem_version'); // no longer used from 1.4.0.0
  
  $users = get_users( 'orderby=meta_value&meta_key=last_name' );
  foreach ( $users as $user ) {
    $mfem_user_id = $user->ID;
    delete_user_meta( $mfem_user_id, 'mfem_user_pw_reset' ); // Delete all meta data generated by the plugin
  }
}
register_uninstall_hook( __FILE__, 'mfem_emergency_management_uninstall' );



// Load the plugin's textdomain to retrieve the translations
function mfem_translations() {
  
  $domain = 'emergency-management';
/* * /
  $mfem_locale = apply_filters( 'plugin_locale', determine_locale(), $domain );
  $mfem_mofile = $domain . '-' . $mfem_locale . '.mo'; // determine the correct .mo-file
  $mfem_mopath = WP_PLUGIN_DIR . '/' . basename( dirname( __FILE__ ) ) . '/languages/';
  unload_textdomain( $domain ); // make sure that the WordPress translations are unloaded
  if ( !load_textdomain( $domain, $mfem_mopath . $mfem_mofile ) ) // Load the very plugin translations (if they exist for $mfem_locale) as the WP SVN version synchronization seems failing...
    load_plugin_textdomain( $domain, false, basename( dirname( __FILE__ ) ) . '/languages/' ); // else: load WordPress plugin translations from /wp-content/languages/plugins/
/* */
  load_plugin_textdomain( $domain, false, basename( dirname( __FILE__ ) ) . '/languages/' ); // load WordPress plugin translations with normal logic
  
}
add_action( 'init', 'mfem_translations' );



function mfem_translate_global_strings () {
  global $thanks_for_using_emergency_management; // translated
  global $mfem_session_tooltip; // translated
  global $you_cannot_reuse_your_old_password; // translated

  $thanks_for_using_emergency_management = __('Thanks a lot for using Emergency Management','emergency-management').'!';
  $mfem_session_tooltip = __('It is recommended to delete the sessions as well','emergency-management').'!';
  $you_cannot_reuse_your_old_password = esc_html__( 'For security reasons, you cannot reuse your old password', 'emergency-management' ).'.';
}
add_action('init', 'mfem_translate_global_strings');



// Change Admin Footer Text for Foodle
function mfem_update_admin_footer_text($content) {
  global $thanks_for_using_emergency_management;
  if ( strpos(get_current_screen()->base, 'mfem-emergency-management') !== false ) {
    remove_all_filters( 'admin_footer_text' );
    return $thanks_for_using_emergency_management.'!';
  }
  else
    return $content;
}
add_filter( 'admin_footer_text', 'mfem_update_admin_footer_text', 1, 1); // Priority 1 to make it work



// Change Admin Footer Update for Foodle
function mfem_update_admin_footer_version($content) {
  if ( strpos(get_current_screen()->base, 'mfem-emergency-management') !== false ) {
    remove_all_filters( 'update_footer' );
    return 'Emergency Management Version '.MFEM_VERSION;
  }
  else
    return $content;
}
add_filter( 'update_footer', 'mfem_update_admin_footer_version', 1, 1); // Priority 1 to make it work



// Enqueue plugin's backend style-file
function mfem_add_stylesheet() {
  $css_url = plugin_dir_url(__FILE__).'styles/emergency-management.css';
  wp_register_style( 'mfem-backend-css', $css_url, array(), MFEM_VERSION, 'all' );
  wp_enqueue_style( 'mfem-backend-css' );
}
add_action( 'admin_enqueue_scripts', 'mfem_add_stylesheet' );



// Enqueue the plugin's javascript/jQuery
function hook_mfem_jQuery_script() {
  $js_url = plugin_dir_url(__FILE__).'js/mfem_jQuery.js';
  wp_register_script( 'mfem_jQuery_script', $js_url, array('jquery'), MFEM_VERSION, false );
  wp_enqueue_script( 'mfem_jQuery_script' );
}
add_action( 'admin_enqueue_scripts', 'hook_mfem_jQuery_script' );  // Just for the admin area



// Add the submenu entry and the help tabs...
function add_emergency_management_menu_items()
{
  $mfem_admin_page = add_submenu_page(
                       'users.php', // related menu
                       'Emergency Management', // page title
                       'EM - Emergency Management', // submenu title
                       'manage_options', // required capability (or role)
                       'mfem-emergency-management', // page slug
                       'mfem_emergency_management_main' // callback function, the main plugin function
                     );
  $mfem_title = __('Reset SALTs & Sessions','emergency-management');
  
  add_action( 'load-'.$mfem_admin_page, 'mfem_add_help' ); // Add the help tabs callback function...
}
add_action('admin_menu','add_emergency_management_menu_items');



// Add the help tabs and sidebar here...
function mfem_add_help () {
  global $version;
  $mfem_screen = get_current_screen();
  
  // Generic explanations
  $id = 'mfem_gen_ex';
  $title = __('Information','emergency-management');
  $content = '<p>'.__('The below \'help\' tabs may be consulted for detailed explanations and hints for their related topic\'s usage.</p><p>
Additional information may be added in the future to display important Q&As.','emergency-management').'</p>';
  $mfem_screen->add_help_tab( array( 
   'id' => $id,            // unique id for the tab
   'title' => $title,      // unique visible title for the tab
   'content' => $content   // actual help text
  ) );
  
  // Explanations for function No. 1
  $id = 'mfem_pwr_sd';
  $title = __('Password Reset & Sessions Delete','emergency-management');
  $content = '<p>'.__('You may select individual roles and users or choose to select "All roles/users".</p><p>
By pressing the button to execute, the password of all selected users will be reset and an eMail will be submitted to each individual with a link to the "Forgot password?" dialogue to define a new password.</p><p>
You may as well choose to just delete the sessions of all selected roles or users, which will not be annouced!.</p><p>
The related users will be logged out immediately with their login cookies and database-stored sessions becoming invalid.</p><p>
For security reasons, a warning query will show up prior final execution.</p><p>
All users with a password reset will be informed by email. The email text can be edited as desired with placeholders being available. Therefore, passwords can only be reset when an email text has been defined.</p><p>
You may use CSS class \'mfem-email-editor\' to control the editor\'s design.','emergency-management').'</p>';
  $mfem_screen->add_help_tab( array( 
   'id' => $id,            // unique id for the tab
   'title' => $title,      // unique visible title for the tab
   'content' => $content   // actual help text
  ) );
  
  // Explanations for function No. 2
  $id = 'mfem_s_ps';
  $title = __('Set Password Strength','emergency-management');
  $content = '<p>'.__('In this area, you can set the required password strength by defining a few parameters.</p><p>
These requirements will apply for all new defined passwords. If you want to ensure this for existing passwords as well, you may want to use 1. above!','emergency-management').'</p>';
  $mfem_screen->add_help_tab( array( 
   'id' => $id,            // unique id for the tab
   'title' => $title,      // unique visible title for the tab
   'content' => $content   // actual help text
  ) );
  
  // Explanations for function No. 3
  $id = 'mfem_s_pex';
  $title = __('Set Password Expiry','emergency-management');
  $content = '<p>'.__('In this area, you can set the required password expiry duration per role.</p><p>
Upon login, the user\'s last password change will be checked against the (shortest) duration for his role(s).</p><p>
And in case the allowed duration is exceeded, he will be guided to the lostpassword dialogue with all his sessions being deleted.</p><p>
In such case, the reuse of the previous password is prohibited.','emergency-management').'</p>';
  $mfem_screen->add_help_tab( array( 
   'id' => $id,            // unique id for the tab
   'title' => $title,      // unique visible title for the tab
   'content' => $content   // actual help text
  ) );
  
  // Explanations for function No. 4
  $id = 'mfem_ks_ren';
  $title = __('KEYs & SALTs Renewal','emergency-management');
  $content = '<p>'.__('This action will locate the "config.php" file and will renew all KEYS & SALTs.</p><p>
KEYs & SALTs build a cryptographic security mechanism, which are used to authenticate the login<br />and certain functions in WordPress.</p><p>
When renewing the KEYs & SALTs, the security environment will change completely with all current users being logged out immediately and all current cookies (and their related database-stored sessions) becoming invalid.</p><p>
The user passwords won\'t change so users can easily login again. A KEYs & SALTs renewal is recommended from time to time.</p><p>
For security reasons, a warning query will show up prior final execution.','emergency-management').'</p>';
  $mfem_screen->add_help_tab( array( 
   'id' => $id,            // unique id for the tab
   'title' => $title,      // unique visible title for the tab
   'content' => $content   // actual help text
  ) );
  
  // Questions & Answers
  $id = 'mfem_q_a';
  $title = __('Questions & Answers','emergency-management');
  $content = '<p>'.__('No Q&As so far...','emergency-management').'</p>';
  $mfem_screen->add_help_tab( array( 
   'id' => $id,            // unique id for the tab
   'title' => $title,      // unique visible title for the tab
   'content' => $content   // actual help text
  ) );
  
  // The help sidebar
  $mfem_sidebar =  '<p style="font-weight:bold;">'.__('Support Hints','emergency-management').'</p>';
  $mfem_sidebar .= '<p>'.__('For plugin information, please consult the','emergency-management');
  $mfem_sidebar .= ' <a href="https://de.wordpress.org/plugins/emergency-management/" target="_blank">';
  $mfem_sidebar .= __('WordPress&nbsp;Plugin&nbsp;Page','emergency-management').'</a>.</p>';
  $mfem_sidebar .= '<p>'.__('In case of questions or issues, you may use the','emergency-management');
  $mfem_sidebar .= ' <a href="https://wordpress.org/support/plugin/emergency-management/" target="_blank">';
  $mfem_sidebar .= __('Plugin&nbsp;Supportpage','emergency-management').'</a> ';
  $mfem_sidebar .= __('or contact the plugin author directly under','emergency-management').' <a href="mailto:plugins@finkenberger.net?subject=Emergency Management Plugin Direct Support">';
  $mfem_sidebar .= __('Direct&nbsp;Support','emergency-management').'</a>.</p>';
  $mfem_sidebar .= '<p>V'.MFEM_VERSION.'</p>';
  $mfem_screen->set_help_sidebar( $mfem_sidebar );
}



// Adding WordPress plugin action links 
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'mfem_emergency_management_add_plugin_action_links' );
function mfem_emergency_management_add_plugin_action_links( $links ) {
  return array_merge(
    array(
      'settings' => '<a href="' . get_bloginfo( 'wpurl' ) . '/wp-admin/users.php?page=mfem-emergency-management">'.__('Settings & Actions','emergency-management').'</a>'
    ),
    $links
  );
}



// Renew the user's meta data password renewal timestamp upon successful user registration
function mfem_save_pw_register_timestamp_to_meta($user_id) {
  $mfem_user_id = $user_id;
  mfem_store_timestamp($mfem_user_id);
}
add_action( 'user_register',  'mfem_save_pw_register_timestamp_to_meta' );
// Renew the user's meta data password renewal timestamp upon successful user password reset by lostpassword dialogue
function mfem_save_pw_reset_timestamp_to_meta($user) {
  $mfem_user_id = $user->ID;
  mfem_store_timestamp($mfem_user_id);
}
add_action( 'after_password_reset', 'mfem_save_pw_reset_timestamp_to_meta' );
// Renew the user's meta data password renewal timestamp upon successful user password reset in the user profile
function mfem_save_pw_profile_change_timestamp_to_meta( $user_id ) {
  $mfem_user_id = $user_id;
  if ( ( ! isset( $_POST['pass1'] ) ) || ( $_POST['pass1'] == '' ) ) return;
  elseif( ! $_POST['pass1'] === $_POST['pass2'] ) return;
  mfem_store_timestamp($mfem_user_id);
}
add_action( 'profile_update', 'mfem_save_pw_profile_change_timestamp_to_meta' );
// Now store the timestamp in user meta
function mfem_store_timestamp($mfem_user_id) {
  if ( ! metadata_exists( 'user', $mfem_user_id, 'mfem_user_pw_reset' ) ) // Use add_user_meta, if it doesn't exist, yet (e.g. registration...)
      add_user_meta( $mfem_user_id, 'mfem_user_pw_reset', wp_date('U'), true ); // Unix timestamp (seconds since Unix Epoch)
  else
    update_user_meta( $mfem_user_id, 'mfem_user_pw_reset', wp_date('U') ); // Unix timestamp (seconds since Unix Epoch)
}



// Enqueue the password check - will only be activated if the option 'mfem_pw_strength' is set
require_once('mfem-enqueue-pw-strength-check.php'); // Installing the password strength check in separate file



function mfem_check_on_login( $redirect_to, $requested_redirect_to, $user ) {

  if ( ( ! isset($user->ID) ) || ( ! get_option('mfem_pw_expiry') ) ) return $redirect_to; // Password expiry is not controlled at this time

  $mfem_user_id = $user->ID;
  $mfem_current_timestamp = wp_date( 'U' ); // Unix timestamp (seconds since Unix Epoch)
  $mfem_last_pw_update = get_user_meta( $mfem_user_id, 'mfem_user_pw_reset', true );

  if ( $mfem_last_pw_update != false ) {
    $mfem_expiries_array = (array)get_option('mfem_pw_expiry'); // Array with all expiring roles and their related expiry in days
    $mfem_user_roles = (array)$user->roles; // Array with all roles the user has
    if ( count(array_intersect(array_keys($mfem_expiries_array), $mfem_user_roles)) <> 0 ) {
      $mfem_days_passed = ( $mfem_current_timestamp - $mfem_last_pw_update ) / ( 60 * 60 * 24 ); // Seconds to days
      $mfem_min_expiry = 1000000; // Will store the minimum expiry
      $mfem_min_role = ''; // Will store the role with the minimum expiry
      foreach( $mfem_user_roles as $mfem_role) {
        if ( ( isset($mfem_expiries_array[$mfem_role]) ) && ( $mfem_expiries_array[$mfem_role] < $mfem_min_expiry ) ) {
          $mfem_min_expiry = $mfem_expiries_array[$mfem_role];
          $mfem_min_role = $mfem_role;
        }
      }
      if ( $mfem_min_expiry < $mfem_days_passed ) {
        //update_option('mfem_test_expiry','role:'.$mfem_min_role.' min:'.$mfem_min_expiry.' passed:'.$mfem_days_passed,'no'); // For test only!!
        return mfem_reset_password_on_expiry($user, $mfem_min_expiry);
      } else {
        return $redirect_to; // The expriry duration has not passed, yet
      }
    } else {
      return $redirect_to; // Expiry duration is not controlled for the user's role(s)
    }
  } else {
    add_user_meta( $mfem_user_id, 'mfem_user_pw_reset', wp_date('U'), true ); // Unix timestamp (seconds since Unix Epoch) // If still not existing (shouldn't happen...)
    return $redirect_to;
  }
}
add_filter( 'login_redirect', 'mfem_check_on_login', 0 , 3 );



// The very function to delete the user's sessions and to define the redirect target
function mfem_reset_password_on_expiry($user, $user_limit) {

  $GLOBALS['current_user'] = $user; // Required to destroy sessions
  wp_destroy_all_sessions();

  return esc_url_raw(
    add_query_arg(
      array(
        'mfem-user-password-expired' => 'mfem-expired',
        'user-limit' => $user_limit
      ),
      wp_lostpassword_url()
    )
  );
}



// Add a message with password expiry information to the lostpassword form, once the password has expired
function lost_password_message( $message ) {

  $status = filter_input( INPUT_GET, 'mfem-user-password-expired', FILTER_SANITIZE_STRING );
  $limit = filter_input( INPUT_GET, 'user-limit', FILTER_SANITIZE_STRING );

  if ( 'mfem-expired' !== $status ) return $message;

  return sprintf(
    '<p id="login_error" style="color:red;">%s.</p><p style="color:green;">%s.</p>',
    sprintf(
      _n(
        'For security reasons, your password must be reset every day',
        'For security reasons, your password must be reset every %d days',
        $limit,
        'emergency-management'
      ),
      $limit
    ),
    esc_html__( 'Please enter your personal data as laid out below and a password reset link will be sent to your eMail address', 'emergency-management' )
  );
}
add_filter( 'login_message', 'lost_password_message', 0, 1 );
add_filter( 'tml_before_form', 'lost_password_message', 0, 1 ); // In case the plugin "Theme My Login" is active



// Make sure that the user will not reuse his last password (no further password history will be checked at this time)
function mfem_validate_password_change( $errors, $user ) {
  global $you_cannot_reuse_your_old_password; // translated

  $new_pass1 = filter_input( INPUT_POST, 'pass1' );
  $new_pass2 = filter_input( INPUT_POST, 'pass2' );

  if ( ! $new_pass1 || ! $new_pass2 || $new_pass1 !== $new_pass2 ) {
    return;
  }

  $mfem_user_roles = (array)$user->roles; // Array with all roles the user has
  $mfem_expiries_array = (array)get_option('mfem_pw_expiry'); // Array with all expiring roles and their related expiry in days
  if ( count(array_intersect(array_keys($mfem_expiries_array), $mfem_user_roles)) == 0 ) return; // Expiry duration is not controlled for the user's role(s)

  $is_same = wp_check_password( $new_pass1, $user->data->user_pass, $user->ID );

  if ( $is_same ) {

//    $errors->add( 'password_already_used', esc_html__( 'For security reasons, you cannot reuse your old password', 'emergency-management' ).'.' );
    $errors->add( 'password_already_used', $you_cannot_reuse_your_old_password );

  }
}
add_action( 'validate_password_reset', 'mfem_validate_password_change', 0, 2 );



// ********************************************************************************************************************************************
// ********************************************************************************************************************************************
// ********************************************************************************************************************************************
// ******************************************************                              ********************************************************
// ******************************************************   The main plugin function   ********************************************************
// ******************************************************                              ********************************************************
// ********************************************************************************************************************************************
// ********************************************************************************************************************************************
// ********************************************************************************************************************************************

function mfem_emergency_management_main() {
  global $mfem_session_tooltip;
  
  // check user capabilities
  if ( ! current_user_can( 'manage_options' ) ) {
    return;
  }
  
  // In case the eMail text was set or updated: check if eMail text is sufficient (min. one word with min. three letters) or needs to be updated and store it as option 'mfem_email_content'
  if ( isset($_POST['mfem_email_content']) ) {
    $mfem_new_email_content = sanitize_text_field(htmlentities(stripslashes($_POST['mfem_email_content'])));
    if ( ( $mfem_new_email_content <> '' ) && ( $mfem_new_email_content === get_option('mfem_email_content') ) ) {
      $mfem_admin_notice  = '<p>&nbsp;</p>';
      $mfem_admin_notice .= __('The email content didn\'t change.','emergency-management');
      $mfem_admin_notice  .= '<p>&nbsp;</p>';
      mfem_pwx_admin_notice__success('<p>'.$mfem_admin_notice.'<p>');
    }
    else {
      if ( ( preg_match('/[a-zA-Z]{3}/', $mfem_new_email_content) )
          && ( update_option('mfem_email_content', $mfem_new_email_content) ) ) {
        $mfem_admin_notice  = '<p>&nbsp;</p>';
        $mfem_admin_notice .= __('The email content was properly stored.','emergency-management');
        $mfem_admin_notice  .= '<p>&nbsp;</p>';
        mfem_pwx_admin_notice__success('<p>'.$mfem_admin_notice.'<p>');
      }
      else {
        $mfem_admin_notice  = '<p>&nbsp;</p>';
        $mfem_admin_notice .= __('The email content wasn\'t stored due to text missing (looking for at least one word with three letters minimum...)!','emergency-management');
        $mfem_admin_notice  .= '<p>&nbsp;</p>';
        mfem_pwx_admin_notice__defined_error('<p>'.$mfem_admin_notice.'<p>');
      }
    }
  }
  
  // If action response/warning is already given - then display it...
  if ( isset( $_REQUEST['mfem_admin_notice'] ) ) {
    $mfem_admin_notice = get_option('mfem_admin_notice');
    delete_option('mfem_admin_notice');
    
    switch($_REQUEST['mfem_admin_notice']) :
    case 'success':
      mfem_pwx_admin_notice__success($mfem_admin_notice);
      break;
    case 'warning':
      mfem_pwx_admin_notice__warning($mfem_admin_notice);
      break;
    case 'error':
      mfem_pwx_admin_notice__defined_error($mfem_admin_notice);
      break;
    case 'prohibited':
      $mfem_admin_notice  = '<p>&nbsp;</p>';
      $mfem_admin_notice .= __('This action is currently prohibited!','emergency-management');
      $mfem_admin_notice .= '<p>&nbsp;</p>';
      mfem_pwx_admin_notice__defined_error($mfem_admin_notice);
    break;
    default:
      mfem_pwx_admin_notice__error();
      break;
    endswitch;
  }
  
  //Get the active tab from the $_GET parameter
  $default_tab = 'pw-reset';
  $tab = isset( $_GET['tab'] ) ? $_GET['tab'] : $default_tab;
  
  // Admin page should be inside a 'wrap': opening div
  echo '<div class="wrap">';
  
  // Display the page title
  echo '<h1 class="mfem-h1">'.esc_html( get_admin_page_title() ).'</h1>';
  echo '<h2 class="mfem-h2">'.__('Provides Means for Your Site\'s Security','emergency-management').'</h2>';
  
  ?><!-- Here are the tabs -->
    <nav class="nav-tab-wrapper">
      <a href="?page=mfem-emergency-management&tab=pw-reset" class="nav-tab <?php if($tab==='pw-reset'): ?>nav-tab-active<?php endif; ?>"><?php _e('Password Reset','emergency-management')?></a>
      <a href="?page=mfem-emergency-management&tab=pw-strength" class="nav-tab <?php if($tab==='pw-strength'): ?>nav-tab-active<?php endif; ?>"><?php _e('Password Strength','emergency-management')?></a>
      <a href="?page=mfem-emergency-management&tab=pw-expiry" class="nav-tab <?php if($tab==='pw-expiry'): ?>nav-tab-active<?php endif; ?>"><?php _e('Password Expiry','emergency-management')?></a>
      <a href="?page=mfem-emergency-management&tab=keys-salts" class="nav-tab <?php if($tab==='keys-salts'): ?>nav-tab-active<?php endif; ?>"><?php _e('KEYs & SALTs','emergency-management')?></a>
    </nav>

    <div class="tab-content">
    <?php switch($tab) :
      case 'pw-reset':
        echo '<p>&nbsp;</p>';
        echo '<h2>'.__('Password Reset & Sessions Delete','emergency-management').'</h2>';
        echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','emergency-management').')</p>';
        echo '<p>&nbsp;</p>';
        mfem_select_pw_reset();
        break;
      case 'pw-strength':
        echo '<p>&nbsp;</p>';
        echo '<h2>'.__('Set Password Strength','emergency-management').'</h2>';
        echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','emergency-management').')</p>';
        echo '<p>&nbsp;</p>';
        mfem_select_pw_strength();
        break;
      case 'pw-expiry':
        echo '<p>&nbsp;</p>';
        echo '<h2>'.__('Set Password Expiry','emergency-management').'</h2>';
        echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','emergency-management').')</p>';
        echo '<p>&nbsp;</p>';
        mfem_select_pw_expiry();
        break;
      case 'keys-salts':
        echo '<p>&nbsp;</p>';
        echo '<h2>'.__('KEYs & SALTs Renewal','emergency-management').'</h2>';
        echo '<p>('.__('Please consult the \'help\' tabs above for detailed explanations and/or use cases.','emergency-management').')</p>';
        echo '<p>&nbsp;</p>';
        echo '<form action="'.esc_attr( admin_url( 'admin-post.php' ) ).'" method="post">';
        echo '<input type="hidden" name="action" value="mfem-saltsform" />';
        echo '<input type="checkbox" id="delete_all_sessions" name="delete_all_sessions" value="delete_all_sessions" checked="checked"><label for="delete_all_sessions">'.__('Delete all user\'s sessions as well (recommended)','emergency-management').'</label><br />';
        echo '<p>&nbsp;</p>';
        echo '<input type="hidden" name="mfemsaltsconfirm" value="yes"/>';
        echo '<input mfem_tooltip="" mfem_tooltip_delay="0" type="submit" id="renew-salts" class="button-primary" value="'.__('Renew KEYs & SALTs...','emergency-management').'"/></form></p>';

        echo '<script type="text/javascript">
          $("#delete_all_sessions").on("click", function(){
          if ( document.getElementById("delete_all_sessions").checked )
            { $("#renew-salts").attr("mfem_tooltip",""); }
          else
            { $("#renew-salts").attr("mfem_tooltip","'.$mfem_session_tooltip.'"); }
          });
        </script>';
        break;
      default:
        echo 'Not covered, yet!';
        break;
    endswitch; ?>
    </div>
  <?php
  
  echo '<p>&nbsp;</p>';
  echo '<p>&nbsp;</p>';
  echo '<p>&nbsp;</p>';
  echo '<hr class="mfem-hr">';
  echo '<p>&nbsp;</p>';
  echo '<p>&nbsp;</p>';
  echo '<p>&nbsp;</p>';
  
  echo '<h2>'.__('If you like this plugin, you could ','emergency-management').'<img src="'.plugin_dir_url( __FILE__ ).'img/PayPal-Logo-2019-kl.jpg" height="16" alt="PayPal">'.__(' your appreciation...','emergency-management').'</h2>';
  echo '<form action="https://www.paypal.com/donate" method="post" target="_top">
        <input type="hidden" name="hosted_button_id" value="AHMXFYUCTVQYU" />
        <input type="image" src="https://www.paypalobjects.com/en_US/DK/i/btn/btn_donateCC_LG.gif" border="0" name="submit" title="PayPal - The safer, easier way to pay online!" alt="Donate with PayPal button" />
        <img alt="" border="0" src="https://www.paypal.com/en_DE/i/scr/pixel.gif" width="1" height="1" />
        </form>';
  echo '<p>&nbsp;</p>';
  echo '<p>&nbsp;</p>';
  echo '<p>&nbsp;</p>';
  echo '<p>&nbsp;</p>';
  echo '<p>&nbsp;</p>';
  echo '<p>&nbsp;</p>';
  
  // Admin page should be inside a 'wrap': closing div
  echo '</div>';
}



function mfem_select_pw_reset() {
  global $mfem_session_tooltip;
  global $wp_roles;
  $users = get_users( 'orderby=meta_value&meta_key=last_name' );
  
  echo '<p><form action="'.esc_attr( admin_url( 'admin-post.php' ) ).'" method="post">';
  echo '<input type="hidden" name="action" value="mfem-pwxform" />';
  echo '<figure class="mfem-block-table"><table style="border-spacing:0px;"><tbody>
        <tr><th style="padding-left:10px; padding-right:10px; text-align:left;">'.__('Roles','emergency-management').'</th><th style="width:1px; background-color:#bbbbbb;"></th><th style="padding-left:10px; padding-right:10px; text-align:left;">'.__('Users','emergency-management').'</th></tr>
        <tr><td colspan="3" style="height:1px; background-color:#bbbbbb;"></td></tr>
        <tr><td style="padding-left:10px; padding-right:10px; vertical-align:top;">';
  echo '<p style="white-space:nowrap; word-wrap:none;"><input type="checkbox" id="all_users" name="roles[]" value="all_users"><label for="all_users">'.__('All roles/users','emergency-management').'</label></p>';
  echo '<p style="height:2px; line-height:0px; padding-top:0px; margin:0px;"></p>';
  foreach ( $wp_roles->role_names as $mfem_key=>$mfem_value) {
    echo '<p style="white-space:nowrap; word-wrap:none; padding:0px; margin: 8px 0px;"><input type="checkbox" id="'.$mfem_key.'" name="roles[]" value="'.$mfem_key.'"><label for="'.$mfem_key.'">'.$mfem_value.'</label></p>';
  }
  echo '</td><td style="background-color:#bbbbbb;"></td><td style="padding-left:10px; padding-right:10px; vertical-align:top;">';
  foreach ( $users as $user ) {
    $mfem_user_id = $user->ID;
    $mfem_user_info = get_userdata($mfem_user_id);
    
    echo '<p style="white-space:nowrap; word-wrap:none; padding:0px; margin: 8px 0px;"><input type="checkbox" id="'.$mfem_user_id.'" name="users[]" value="'.$mfem_user_id.'"><label for="'.$mfem_user_id.'">'.$mfem_user_info->user_login.'</label></p>';
  }
  echo '</td></tr></tbody></table></figure>';
  echo '<p>&nbsp;</p>';
  echo '<input type="checkbox" id="delete_sessions" name="delete_sessions" value="delete_sessions" checked="checked"><label for="delete_sessions">'.__('With the password reset, delete all related user\'s sessions as well (recommended)','emergency-management').'</label><br />';
  echo '<p>&nbsp;</p>';
  if (!get_option("mfem_email_content") ) {
    $mfem_type = 'button';
    echo '<div style="color:#500404;">('.__('The eMail content has not been set, yet. You will not be able to start the password reset at this time!','emergency-management').')</div><br />';
  } else $mfem_type = 'submit';
  echo '<input mfem_tooltip="" mfem_tooltip_delay="0" id="mfem-pw-reset" type="'.$mfem_type.'" class="button-primary" value="'.__('Reset passwords...','emergency-management').'" />';
  echo '<p>&nbsp;</p>';
  echo '<input type="submit" class="button-primary" name="just_delete_sessions" value="'.__('Just delete all related sessions...','emergency-management').'" /></form></p>';
  echo '<p>&nbsp;</p>';
  
  echo '<p><table><tr><th>'.__('eMail Content','emergency-management').'</th></tr></table></p>';
  
  $settings = array(
    'wpautop' => false,
    'media_buttons' => true,
    'default_editor' => '',
    'drag_drop_upload' => true,
    'textarea_rows' => 30,
    'tabindex' => 5,
    'editor_css' => '',
    'editor_class' => 'mfem-email-editor',
    'teeny' => false,
    'tinymce' => true,
    'quicktags' => true
  );
  $mfem_email_content = '';
  if ( get_option('mfem_email_content') ) $mfem_email_content = html_entity_decode(get_option('mfem_email_content'));
  echo '<form name="mfem_editor" action="'.admin_url( 'admin.php?page=mfem-emergency-management' ).'" method="post">';
  echo '<div style="width:90%;">';
  wp_editor($mfem_email_content, 'mfem_email_content', $settings);
  echo '</div>';
  echo '<br /><strong>'.__('Available placeholders','emergency-management').':</strong><br />{first-name}&nbsp;&nbsp;{last-name}&nbsp;&nbsp;{nick-name}&nbsp;&nbsp;{user-login}&nbsp;&nbsp;{user-email}&nbsp;&nbsp;{password-forgot-url}&nbsp;&nbsp;{password-forgot-link}';
  echo '<p>&nbsp;</p>';
  echo '<input type="submit" class="button-primary" value="'.__('Update email content...','emergency-management').'" /></form>';

  echo '<script type="text/javascript">
    $("#delete_sessions").on("click", function(){
      if ( document.getElementById("delete_sessions").checked )
        { $("#mfem-pw-reset").attr("mfem_tooltip",""); }
      else
        { $("#mfem-pw-reset").attr("mfem_tooltip","'.$mfem_session_tooltip.'"); }
    });
  </script>';
}



function mfem_select_pw_strength() {

  if ( !get_option('mfem_pw_strength') ) {
    $not_set = '<div style="color:#500404;">('.__('Password strength has not been set and is therefore not being monitored!','emergency-management').')</div>';
    $digit_checked = 'checked="checked"';
    $uppercase_char_checked ='';
    $lowercase_char_checked = 'checked="checked"';
    $special_char_checked = '';
    $double_special_checked ='';
    $pw_min = 8;
    $pw_max = 12;
  } else {
    $not_set = '<div style="color:#045004;">('.__('Password strength has been set and is being monitored!','emergency-management').')</div>';
    $mfem_pw_strenght_store = (array)get_option('mfem_pw_strength');
    $mfem_pw_content = (array)$mfem_pw_strenght_store[0];
    $digit_checked = ( in_array( 'digit', $mfem_pw_content) ) ? 'checked="checked"' : '';
    $uppercase_char_checked = ( in_array( 'uppercase_char', $mfem_pw_content) ) ? 'checked="checked"' : '';
    $lowercase_char_checked = ( in_array( 'lowercase_char', $mfem_pw_content) ) ? 'checked="checked"' : '';
    $special_char_checked = ( in_array( 'special_char', $mfem_pw_content) ) ? 'checked="checked"' : '';
    $double_special_checked = ( in_array( 'double_special', $mfem_pw_content) ) ? 'checked="checked"' : '';
    $pw_min = $mfem_pw_strenght_store[1];
    $pw_max = $mfem_pw_strenght_store[2];
  }
  
  $mfem_special_chars = "! \" # $ % & ' ( ) * + , . / : ; < = > ? @ [ \ ] ^ ` { | } ~ ]";
  $mfem_min_warning = __('WARNING:\nLess than 8 characters are secure enough!','emergency-management');
  $mfem_max_warning = __('WARNING:\nThe WordPress database limits\npasswords to 20 characters in length!','emergency-management');
  
  echo ' 
  <script type=\'text/javascript\'>
    jQuery(document).ready(function() {
      jQuery(window).keydown(function(event){
        if(event.keyCode == 13) {
          mfem_this = document.activeElement;
          if ( ( mfem_this.value < 8 ) || ( mfem_this.value > 20 ) ) {
            if ( mfem_this.value < 8 ) {
              alert("'.$mfem_min_warning.'");
              mfem_this.value = 8;
            } else {
              alert("'.$mfem_max_warning.'");
              mfem_this.value = 20;
            }
            event.preventDefault();
            return false;
          }
          return true;
        }
      });
    });
  </script>'; // Prevent an uncontrolled default form submit by the <RETURN> key.
  
  echo '<p><form name="strength_form" id="strength_form" action="'.esc_attr( admin_url( 'admin-post.php' ) ).'" method="post">';
  echo '<input type="hidden" name="action" value="mfem-pwstrengthform1" />';
  echo '<input margin-left:auto; margin-right:auto;" type="checkbox" id="digit" name="pw_content[]" value="digit" '.$digit_checked.' />&nbsp;<label for ="digit">'.__('Minimum one digit','emergency-management').' (*)</label><br /><br />';
  echo '<input type="checkbox" id="uppercase_char" name="pw_content[]" value="uppercase_char" '.$uppercase_char_checked.' />&nbsp;<label for ="uppercase_char">'.__('Minimum one uppercase character','emergency-management').'</label><br /><br />';
  echo '<input type="checkbox" id="lowercase_char" name="pw_content[]" value="lowercase_char" '.$lowercase_char_checked.' />&nbsp;<label for ="lowercase_char">'.__('Minimum one lowercase character','emergency-management').' (*)</label><br /><br />';
  echo '<input type="checkbox" id="special_char" name="pw_content[]" value="special_char" '.$special_char_checked.' />&nbsp;<label for ="special_char">'.__('Minimum one special character taken from','emergency-management').' '.$mfem_special_chars.'</label><br /><br />';
  echo '<input type="checkbox" id="double_special" name="pw_content[]" value="double_special" '.$double_special_checked.' />&nbsp;<label for ="double_special">'.__('Minimum one special char pair (same or different ones) taken from','emergency-management').' '.$mfem_special_chars.'</label><br /><br />';
  echo __('Underscores (_) and hypens (-) are allowed anyway - spaces are not allowed!','emergency-management').'<br /><br />';
  echo '<input style="text-align:center; width:60px;" type="text" id="pw_min" name="pw_min" value="'.$pw_min.'" onSubmit="return false;" onkeyup="this.value=this.value.replace(/\D/, \'\');" onfocusout="if (this.value<8) { alert(\''.$mfem_min_warning.'\'); this.value=8; this.focus(); } else if (this.value>20) { alert(\''.$mfem_max_warning.'\'); this.value=20; this.focus(); }" />&nbsp;&nbsp;<label for="pw_min">'.__('Minimum length','emergency-management').' (*8)</label><br /><br />';
  echo '<input style="text-align:center; width:60px;" type="text" id="pw_max" name="pw_max" value="'.$pw_max.'" onSubmit="return false;" onkeyup="this.value=this.value.replace(/\D/, \'\');" onfocusout="if (this.value<8) { alert(\''.$mfem_min_warning.'\'); this.value=8; this.focus(); } else if (this.value>20) { alert(\''.$mfem_max_warning.'\'); this.value=20; this.focus(); }" />&nbsp;&nbsp;<label for="pw_max">'.__('Maximum length','emergency-management').' (*12)&nbsp;&nbsp;&nbsp;→ WordPress '.__('limit','emergency-management').': 20!</label><br /><br />';
  echo '<p>'.$not_set.'</p>';
  echo '<input type="submit" class="button-primary" name="mfem-set-pw-strenght" value="'.__('Set password strength...','emergency-management').'"/> (*) = '.__('weak default','emergency-management').'</form></p>';
  echo '<p>&nbsp;</p>';
  echo '<p><form name="strength_cancel_form" id="strength_cancel_form" action="'.esc_attr( admin_url( 'admin-post.php' ) ).'" method="post">';
  echo '<input type="hidden" name="action" value="mfem-pwstrengthform2" />';
  echo '<input type="submit" class="button-primary" style="background-color:darkred;" name="mfem-cancel-pw-strenght" value="'.__('Cancel password strength...','emergency-management').'"/></form></p>';
}



function mfem_select_pw_expiry() {
  global $wp_roles;
  
  $roles_expiry = array();
  if ( get_option('mfem_pw_expiry') ) $roles_expiry = get_option('mfem_pw_expiry');
  
  $mfem_min_warning = __('WARNING:\nAn expiry less than 10 days is not recommended!','emergency-management');

  echo ' 
  <script type=\'text/javascript\'>
    jQuery(document).ready(function() {
      jQuery(window).keydown(function(event){
        if(event.keyCode == 13) {
          mfem_this = document.activeElement;
          if ( ( mfem_this.value < 10 ) && ( mfem_this.value != "" ) ) {
            alert("'.$mfem_min_warning.'");
            mfem_this.value = 10;
            event.preventDefault();
            return false;
          }
          return true;
        }
      });
    });
  </script>'; // Prevent an uncontrolled default form submit by the <RETURN> key.
  
  echo '<p><form action="'.esc_attr( admin_url( 'admin-post.php' ) ).'" method="post">';
  echo '<input type="hidden" name="action" value="mfem-pwexpiryform" />';
  echo '<input type="hidden" name="roles[]" value="mfem_dummy" />'; // Just to have the array exist
  echo '<figure class="mfem-block-table"><table style="border-spacing:0px;"><tbody>
        <tr><th style="padding-left:10px; text-align:left;">'.__('Roles','emergency-management').'</th><th style="padding-left:10px; padding-right:10px; text-align:right;">'.__('Expiry [days]','emergency-management').'</th></tr>
        <tr><td colspan="2" style="height:1px; background-color:#bbbbbb;"></td></tr>';
  foreach ( $wp_roles->role_names as $mfem_key=>$mfem_value) {
    $mfem_checked = ( isset($roles_expiry[$mfem_key]) ) ? ' checked="checked"' : '';
    $mfem_expiry = ( isset($roles_expiry[$mfem_key]) ) ? $roles_expiry[$mfem_key] : '';
    echo '<tr>';
    echo '<td style="padding:0px 0px 0px 10px; margin:0px;"><p style="white-space:nowrap; word-wrap:none; padding:0px; margin:0px;"><input type="checkbox" id="'.$mfem_key.'" name="roles[]" value="'.$mfem_key.'"'.$mfem_checked.'/><label for="'.$mfem_key.'">'.$mfem_value.'</label></p></td>';
    echo '<td style="padding:0px 10px 0px 0px; margin:0px; text-align:right;"><p style=" padding:0px; margin: 6px 0px;"><input type="text" style="display:inline-block; padding:0px 10px; line-height:0px;" size="3" maxlength="3" name="roles_expiry['.$mfem_key.']" placeholder="90" value="'.$mfem_expiry.'" onkeyup="this.value=this.value.replace(/\D/, \'\');" onfocusout="if ( (this.value < 10) && (this.value != \'\') ) { alert(\''.$mfem_min_warning.'\'); this.value=10; this.focus(); }" /></p></td>';
    echo '</tr>';
  }
  echo '</tbody></table></figure>';
  echo '<p>&nbsp;</p>';
  echo '<input type="submit" class="button-primary" name="set_pw_expiry" value="'.__('Set password exiry...','emergency-management').'" /></form></p>';
}



require_once('mfem-password-reset.php'); // request-handler triggered by the warning form to reset the passwords and send the emails in separate file
add_action( 'admin_post_mfem-pwxform', 'mfem_warning_pwxform_handler' ); // No admin_post_nopriv_... as we'll stay in the admin or registered users area
add_action( 'admin_post_mfem-pwxconfirm', 'mfem_reset_passwords' );  // No admin_post_nopriv_... as we'll stay in the admin or registered users area

require_once('mfem-salts-renewal.php'); // request-handler triggered by the warning form to renew the SALTs in separate file
add_action( 'admin_post_mfem-saltsform', 'mfem_warning_saltsform_handler' ); // No admin_post_nopriv_... as we'll stay in the admin or registered users area
add_action( 'admin_post_mfem-saltsconfirm', 'mfem_renew_salts' );  // No admin_post_nopriv_... as we'll stay in the admin or registered users area

add_action( 'admin_post_mfem-pwstrengthform1', 'mfem_save_pw_strength_handler' ); // No admin_post_nopriv_... as we'll stay in the admin or registered users area
add_action( 'admin_post_mfem-pwstrengthform2', 'mfem_save_pw_strength_handler' ); // No admin_post_nopriv_... as we'll stay in the admin or registered users area

add_action( 'admin_post_mfem-pwexpiryform', 'mfem_pw_expiry_handler' ); // No admin_post_nopriv_... as we'll stay in the admin or registered users area



// The success notice as admin_notice...
function mfem_pwx_admin_notice__success($mfem_success_message) {
  ?>
  <div class="notice notice-success is-dismissible">
    <?php echo $mfem_success_message; ?>
  </div>
  <?php
}



// The defined error notice(s) as admin_notice...
function mfem_pwx_admin_notice__defined_error($mfem_error_message) {
  ?>
  <div class="notice notice-error is-dismissible">
    <?php echo _e('ERROR','emergency-management').': '.$mfem_error_message; ?>
  </div>
  <?php
}



// The undefined error notice as admin_notice...
function mfem_pwx_admin_notice__error() {
  ?>
  <div class="notice notice-error is-dismissible">
    <h2 class="mfem-h2"><?php echo _e( 'ERROR: The action was most probably not properly performed!', 'emergency-management' ); ?></h2>
  </div>
  <?php
}



// The last-chance-cancel warning(s) as admin_notice...
function mfem_pwx_admin_notice__warning($mfem_warning_message) {
  ?>
  <div class="notice notice-warning is-dismissible">
    <?php echo $mfem_warning_message; ?>
  </div>
  <?php
}



function mfem_warning_pwxform_handler() {
  $mfem_warning_message  = '<h2 class="mfem-h2">'.__('Are you sure you want to proceed?','emergency-management').'</h2>';
  if ( isset($_POST['just_delete_sessions']) )
    $mfem_warning_message .= __('About to delete the related user\'s sessions, which will log them off!','emergency-management');
  else
    $mfem_warning_message .= __('About to reset the related user\'s passwords (= log-off) and to submit email notifications<br />to the users, asking them to renew their passwords!','emergency-management');
  $mfem_warning_message .= '<p>&nbsp;</p>';
  $mfem_warning_message .= '<form action="'.esc_attr( admin_url( 'admin-post.php' ) ).'" method="post">';
  $mfem_warning_message .= '<input type="hidden" name="action" value="mfem-pwxconfirm" />';
  if ( isset($_POST['roles']) ) {
    foreach( $_POST['roles'] as $role) $mfem_warning_message .= '<input type="hidden" id="roles" name="roles[]" value="'.sanitize_text_field($role).'" />';
    $mfem_roles_selection = wp_kses( implode("<br />",$_POST['roles']), array('br' => array(),) );
  } else $mfem_roles_selection = __('none','emergency-management');
  if ( isset($_POST['users']) ) {
    $mfem_users_selection_array = array();
    foreach( $_POST['users'] as $user) {
      $user = sanitize_text_field($user);
      $mfem_warning_message .= '<input type="hidden" id="users" name="users[]" value="'.$user.'" />';
      $mfem_users_selection_array[] = get_userdata($user)->user_login;
    }
    $mfem_users_selection = implode('<br />', $mfem_users_selection_array);
  } else $mfem_users_selection = __('none','emergency-management');
  $mfem_warning_message .= '<strong>'.__('Roles selected:','emergency-management').'</strong><br />'.$mfem_roles_selection.'<br /><br />';
  $mfem_warning_message .= '<strong>'.__('Users selected (plus those addressed by the selected roles):','emergency-management').'</strong><br />'.$mfem_users_selection;
  $mfem_warning_message .= '<p>&nbsp;</p>';
  if ( ( isset($_POST['delete_sessions']) ) && ( !isset($_POST['just_delete_sessions']) ) ) {
    $mfem_warning_message .= '<input type="hidden" name="delete_sessions" value="'.$_POST['delete_sessions'].'" />';
    $mfem_warning_message .= __('All related user\'s sessions will as well be deleted!','emergency-management');
    $mfem_warning_message .= '<p>&nbsp;</p>';
  }
  if ( isset($_POST['just_delete_sessions']) )
    $mfem_warning_message .= '<input type="hidden" name="just_delete_sessions" value="'.$_POST['just_delete_sessions'].'" />';
  
  $mfem_warning_message .= '<input type="submit" class="button-primary" name="mfempwxconfirm" value="'.__('Yes','emergency-management').'"/>';
  $mfem_warning_message .= '</form>';
  $mfem_warning_message .= '<p>&nbsp;</p>';
  add_option( 'mfem_admin_notice', $mfem_warning_message, '', 'no' );
  
  nocache_headers();
  wp_safe_redirect( esc_url_raw( add_query_arg(
    array(
      'mfem_admin_notice' => 'warning',
    ),
    admin_url( 'admin.php?page=mfem-emergency-management&tab=pw-reset' ) 
  ) ) );
  
  status_header(200); // everything is OK
  exit();             // request-handler should always exit...
  
}



function mfem_warning_saltsform_handler_pre() {
  // check user capabilities
  if ( ! current_user_can( 'manage_options' ) ) {
    return;
  }
  
  $_POST['delete_all_sessions'] = true;
  mfem_warning_saltsform_handler();
}
function mfem_warning_saltsform_handler() {
  $mfem_warning_message  = '<h2 class="mfem-h2">'.__('Are you sure you want to proceed?','emergency-management').'</h2>';
  $mfem_warning_message .= '<form action="'.esc_attr( admin_url( 'admin-post.php' ) ).'" method="post">';
  $mfem_warning_message .= '<input type="hidden" name="action" value="mfem-saltsconfirm" />';
  $mfem_warning_message .= __('About to renew the SALTs, which will result in all users being logged out!','emergency-management');
  $mfem_warning_message .= '<p>&nbsp;</p>';
  if ( isset($_POST['delete_all_sessions']) ) {
    $mfem_warning_message .= '<input type="hidden" name="delete_all_sessions" value="'.sanitize_text_field($_POST['delete_all_sessions']).'" />';
    $mfem_warning_message .= __('All user\'s sessions will as well be deleted!','emergency-management');
    $mfem_warning_message .= '<p>&nbsp;</p>';
  }
  $mfem_warning_message .= '<input type="submit" class="button-primary" name="mfemsaltsconfirm" value="'.__('Yes','emergency-management').'"/>';
  $mfem_warning_message .= '</form>';
  $mfem_warning_message .= '<p>&nbsp;</p>';
  add_option( 'mfem_admin_notice', $mfem_warning_message, '', 'no' );
  
  nocache_headers();
  wp_safe_redirect( esc_url_raw( add_query_arg(
    array(
      'mfem_admin_notice' => 'warning',
    ),
    admin_url( 'admin.php?page=mfem-emergency-management&tab=keys-salts' ) 
  ) ) );
  
  status_header(200); // everything is OK
  exit();             // request-handler should always exit...
  
}



function mfem_save_pw_strength_handler() {
  
  $mfem_info_message  = '<h2 class="mfem-h2">'.__('Password Strength Results','emergency-management').'</h2>';
  
  if ( isset($_POST['mfem-set-pw-strenght']) ) {
    $mfem_pw_min = ( isset($_POST['pw_min']) ) ? (int)sanitize_text_field($_POST['pw_min']) : (int)10;
    $mfem_pw_max = ( isset($_POST['pw_max']) ) ? (int)sanitize_text_field($_POST['pw_max']) : (int)20;
    $mfem_pw_content = ( isset($_POST['pw_content']) ) ? explode('#', sanitize_text_field(implode('#', $_POST['pw_content']))) : array();
    if ( $mfem_pw_max < $mfem_pw_min ) { $mfem_pw_tmp = $mfem_pw_min; $mfem_pw_min = $mfem_pw_max; $mfem_pw_max = $mfem_pw_tmp; }
    $mfem_pw_strenght_store = array();
    $mfem_pw_strenght_store[] = $mfem_pw_content;
    $mfem_pw_strenght_store[] = $mfem_pw_min;
    $mfem_pw_strenght_store[] = $mfem_pw_max;
    
    if ( !get_option('mfem_pw_strength') ) {
      $ok = add_option ('mfem_pw_strength', $mfem_pw_strenght_store, '','yes');
    }
    else {
      $mfem_temp = get_option('mfem_pw_strength');
      if ( $mfem_temp === $mfem_pw_strenght_store ) {
        $mfem_info_message .= '<p>'.__('No update required!','emergency-management').'</p>';
        $ok = true;
      }
      else $ok = update_option ('mfem_pw_strength', $mfem_pw_strenght_store);
    }
    if ( $ok ) {
      $mfem_info_message .= __('Password strenght has been stored successfully!','emergency-management');
      $feedback = 'success';
    } else {
      $mfem_info_message .= __('Error while trying to store the Password strenght!','emergency-management');
      $feedback = 'error';
    }
  } else if ( isset($_POST['mfem-cancel-pw-strenght']) ) {
    delete_option('mfem_pw_strength');
    $mfem_info_message .= __('The password strength filter has been removed!','emergency-management');
    $feedback = 'success';
  } else {
    $mfem_info_message  .= __('Hmmm... - minimum one option should be checked!','emergency-management');
    $feedback = 'warning';
  }
  $mfem_info_message .= '<p>&nbsp;</p>';
  add_option( 'mfem_admin_notice', $mfem_info_message, '', 'no' ); // using add_option since this plugin will delete the option as soon as it is used...
  
  nocache_headers();
  wp_safe_redirect( esc_url_raw( add_query_arg(
    array(
      'mfem_admin_notice' => $feedback,
    ),
    admin_url( 'admin.php?page=mfem-emergency-management&tab=pw-strength' ) 
  ) ) );
  
  status_header(200); // everything is OK
  exit();             // request-handler should always exit...
  
}



function mfem_pw_expiry_handler() {
  global $wp_roles;
  
  $mfem_info_message  = '<h2 class="mfem-h2">'.__('Password Expiry Results','emergency-management').'</h2>';
  
  if ( isset($_POST['roles']) ) {
    unset($_POST['roles'][0]); // 'mfem_dummy'
    $selected_roles = array();
    foreach( $_POST['roles'] as $role) {
      $role = sanitize_text_field( $role );
      if ( ( isset($_POST['roles_expiry'][$role]) ) && ( $_POST['roles_expiry'][$role] != '' ) )
        $expiry_days = sanitize_text_field( $_POST['roles_expiry'][$role] );
      else
        $expiry_days = 90;
      $selected_roles[$role] = $expiry_days;
      $mfem_info_message .= '<p>'.$wp_roles->role_names[$role].' - '.$expiry_days.'</p>';
    }
    update_option('mfem_pw_expiry', $selected_roles, 'yes');
    $feedback = 'success';
    if ( count($selected_roles) == 0 )
      $mfem_info_message .= '<p style="color:darkred;">'.__('No roles\' passwords are being watched','emergency-management').'!</p>';;
    $mfem_info_message .= '<p>&nbsp;</p>';
    $mfem_info_message .= '<p>'.__('Password expiry data was properly set','emergency-management').'.</p>';;
  } else {
    $feedback = 'unknown';
  }
  $mfem_info_message .= '<p>&nbsp;</p>';
  add_option( 'mfem_admin_notice', $mfem_info_message, '', 'no' ); // using add_option since this plugin will delete the option as soon as it is used...
  
  nocache_headers();
  wp_safe_redirect( esc_url_raw( add_query_arg(
    array(
      'mfem_admin_notice' => $feedback,
    ),
    admin_url( 'admin.php?page=mfem-emergency-management&tab=pw-expiry' ) 
  ) ) );
  
  status_header(200); // everything is OK
  exit();             // request-handler should always exit...
  
}


