<?php
/**
 * Have passwords of a defined group expired and have them set it new, based on hints in an eMail.
 * Author: Michael Finkenberger
 * @since V1.0.0.0
 * Last change in plugin version: V1.4.0.0
 * Date: 18.05.2021
 * Tested with plugin version: V1.4.0.0
 */

if(!defined('ABSPATH')) die();// no direct access



// Global definition
$mfem_placeholder_array = array();



function mfem_reset_passwords() {
  
  global $mfem_placeholder_array;
  
  if(current_user_can('manage_options')) { // One must be authorized to do this...
    $mfem_result_message  = '<h2 class="mfem-h2">'.__('Actions Performed','emergency-management').'</h2>';
    
    $pwx_users = get_users( 'orderby=meta_value&meta_key=last_name' );
    
    $nothing_done = true;
    
    if ( (get_option("mfem_email_content") ) || ( isset($_POST['just_delete_sessions']) ) ){
      foreach ( $pwx_users as $pwx_user ) {
        $pwx_user_id = $pwx_user->ID;
        $pwx_user_info = get_userdata($pwx_user_id);
        $pwx_user_meta = get_user_meta($pwx_user_id);
        $pwx_user_roles = (array)$pwx_user->roles;
        
        $mfem_placeholder_array = array(
          '{first-name}'           => $pwx_user_info->first_name,
          '{last-name}'            => $pwx_user_info->last_name,
          '{nick-name}'            => $pwx_user_info->nickname,
          '{user-login}'           => $pwx_user_info->user_login,
          '{user-email}'           => $pwx_user_info->user_email,
          '{password-forgot-url}'  => wp_lostpassword_url(),
          '{password-forgot-link}' => '<a href = "'.wp_lostpassword_url().'">'.__('lost password','emergency-management').'</a>'
        );
        $post_roles = ( isset($_POST['roles']) ) ? (array)$_POST['roles'] : array();
        $post_users = ( isset($_POST['users']) ) ? (array)$_POST['users'] : array();
        if ( ( in_array( 'all_users', $post_roles ) ) || ( array_intersect( $post_roles, $pwx_user_roles ) ) || (  in_array( $pwx_user_id, $post_users ) ) ) {
          $mfem_result_message .= emergency_password_reset($pwx_user_id, $pwx_user_info, isset($_POST['delete_sessions']), isset($_POST['just_delete_sessions']));
          $nothing_done = false;
        }
      }
      if ( $nothing_done ) $mfem_result_message .= '<p>'.__('Nothing to do...','emergency-management').'</p>';
      $mfem_result_message .= '<p>&nbsp;</p>';
      $feedback = 'success';  
    } else {
    $mfem_result_message .= '<p>'.__('eMail content has not been set, yet - Nothing done!','emergency-management').'</p>';
    $mfem_result_message .= '<p>&nbsp;</p>';
    $feedback = 'error';
    }
  } else $feedback = 'prohibited';
  
  add_option( 'mfem_admin_notice', $mfem_result_message, '', 'no' );
  
  nocache_headers();
  wp_safe_redirect( esc_url_raw( add_query_arg(
    array(
      'mfem_admin_notice' => $feedback,
    ),
    admin_url( 'admin.php?page=mfem-emergency-management&tab=pw-reset' ) 
  ) ) );
  
  status_header(200); // everything is OK
  exit();             // request-handler should always exit...
  
}

// The real stuff...
function emergency_password_reset($ID, $user_info, $delete_sessions, $just_delete_sessions) {
  
  global $mfem_placeholder_array;
  
  if(current_user_can('manage_options')) { // just to be sure...
    if ( !$just_delete_sessions ) {
      $password = wp_generate_password();
      $message = html_entity_decode(get_option("mfem_email_content"));
      
      foreach ( $mfem_placeholder_array as $placeholder => $ph_content ) {
        $message = str_replace($placeholder, $ph_content, $message);
      }
      
      wp_set_password( $password, $ID );                    // Now the password is done...
      
      add_filter( 'wp_mail_from_name', function( $adminname ) { return explode('@', get_option('admin_email'))[0]; }, 10 );
      add_filter( 'wp_mail_from', function( $email ) {return get_option('admin_email');}, 10 );
      add_filter( 'wp_mail_content_type','emergency_password_reset_set_html_mail_content_type', 10 );
      $ok = wp_mail( $user_info->user_email,__('Password reset for','emergency-management').' '.get_option('blogname'),$message );
      
      // Remove all those filters while not needed to avoid any conflict with other plugins or themes...
      remove_filter( 'wp_mail_from_name', function( $adminname ) {return explode('@', get_option('admin_email'))[0];}, 10 );
      remove_filter( 'wp_mail_from', function( $email ) {return get_option('admin_email');}, 10 );
      remove_filter( 'wp_mail_content_type','emergency_password_reset_set_html_mail_content_type', 10 );
    }
    
    if ( ( $delete_sessions ) || ( $just_delete_sessions ) ) {
      $sessions = WP_Session_Tokens::get_instance( $ID );
      $sessions->destroy_all();
    };                                                    // ... and the user's sessions as well ...
    
    if ( !$just_delete_sessions ) {
      if ( $delete_sessions ) $del_sess = ' - '.__('the user\'s sessions were deleted','emergency-management'); else $del_sess = '';
      if ( $ok) {
        $return = '<p>'.__('Password has been reset and email was sent for','emergency-management').': '.$user_info->user_login.$del_sess.'</p>';
        return $return;
      } else {
        return '<p>'.__('Password has been reset - but an an email-send-ERROR was received for','emergency-management').': '.$user_info->user_login.$del_sess.'</p>';
      }
    } else {
        return '<p>'.__('All sessions have been reset for','emergency-management').': '.$user_info->user_login.'</p>';
    }
  }
}



// Setting the email type to HTML...
function emergency_password_reset_set_html_mail_content_type() {
    return 'text/html';
}


