<?php
/**
 * Have passwords of a defined group expired and have them set it new, based on hints in an eMail.
 * Author: Michael Finkenberger
 * @since V1.0.0.0
 * Last change in plugin version: V1.4.0.0
 * Date: 18.05.2021
 * Tested with plugin version: V1.4.0.0
 */

if(!defined('ABSPATH')) die();// no direct access



function mfem_renew_salts() {
  if(current_user_can('manage_options')) { // One must be authorized to do this...
    $mfem_result_message  = '<h2 class="mfem-h2">'.__('Actions Performed for SALTs','emergency-management').'</h2>';
    
    $nothing_done = true;
    
    $config_file     = ABSPATH . 'wp-config.php';
    if ( file_exists( $config_file ) && is_writable( $config_file ) ) {
      
      $mfem_result_message .= '<p>'.__('Located the "wp-config.php" file and it is writable.','emergency-management').'</p>';
       
      $old_salts = array('AUTH_KEY' => AUTH_KEY,
                         'SECURE_AUTH_KEY' => SECURE_AUTH_KEY,
                         'LOGGED_IN_KEY' => LOGGED_IN_KEY,
                         'NONCE_KEY' => NONCE_KEY,
                         'AUTH_SALT' => AUTH_SALT,
                         'SECURE_AUTH_SALT' => SECURE_AUTH_SALT,
                         'LOGGED_IN_SALT' => LOGGED_IN_SALT,
                         'NONCE_SALT' => NONCE_SALT
                        );
      $new_salts = array();
      
      $mfem_result_message .= '<p>'.__('Defining the new keys...','emergency-management').'</p>';
      
      $chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()-_ []{}<>~`+=,.;:/?|';
      foreach($old_salts as $key => $value) {
        $max = strlen($chars) - 1;
        $salt = '';
        for ( $j = 0; $j < 64; $j++ )
          $salt .= substr( $chars, random_int( 0, $max ), 1 );
        $new_salts[$key] = $salt;
      }
      
      $mfem_result_message .= '<p>'.__('Writing the new keys into a temporary file...','emergency-management').'</p>';
      
      $reading_config  = fopen( $config_file, 'r' );
      $tmp_config_file = ABSPATH . 'wp-config-tmp.php';
      $writing_config = fopen( $tmp_config_file, 'w' );
      $count = 0;

      $mfem_cur_date = wp_date('d.M.Y//H:i');
      $credential1 = "/* These eight KEYs & SALTs were generated on ".$mfem_cur_date." by \"Emergency Management\". */\n";
      $credential2 = "/* Author: Michael Finkenberger  -  Thanks a lot for using the plugin and for contributing! */\n";
      $credential3 = "/* End of the eight KEYs & SALTs generated by the fully free \"Emergency Management\" plugin. */\n";
      $credcheck1 = substr($credential1, 0, 40);
      $credcheck2 = substr($credential2, 0, 40);
      $credcheck3 = substr($credential3, 0, 40);
      while ( ! feof( $reading_config ) ) {
        $line = fgets( $reading_config );
        foreach ( $old_salts as $salt_key => $salt_value ) {
          if ( stristr( $line, $salt_value ) ) {
            $count += 1;
            if ( $count == 1 ) fputs( $writing_config, $credential1.$credential2);
            $line = str_replace($salt_value,$new_salts[ $salt_key ],$line);
          }
          $linecheck = substr($line, 0, 40);
          if ( ( $linecheck == $credcheck1 ) || ( $linecheck == $credcheck2 ) || ( $linecheck == $credcheck3 ) ) $line = '';
        }
        fputs( $writing_config, $line );
        if ( $count == 8 ) {
          $count = 0;
          fputs( $writing_config, $credential3 );
        }
      }
      fclose( $reading_config );
      fclose( $writing_config );
      
      if ( isset($_POST['delete_all_sessions']) ) {
        $mfem_result_message .= '<p>'.__('Deleting all user\'s sessions...','emergency-management').'</p>';
        $users = get_users( 'orderby=meta_value&meta_key=last_name' );
        foreach ( $users as $user ) {
          $sessions = WP_Session_Tokens::get_instance( $user->ID );
          $sessions->destroy_all(); // This is the very action for the sessions!
        }
      }
      
      $mfem_result_message .= '<p><strong>'.__('Done ... so you were logged out','emergency-management').' ;-)!</strong></p>';
      $mfem_result_message .= '<p>&nbsp;</p>';
      
      rename( $tmp_config_file, $config_file ); // This is the very action for the KEYs & SALTs!
      
      $feedback = 'success';
    } else {
      $feedback = 'error';
      $mfem_result_message = '<p>&nbsp;</p>';
      $mfem_result_message .= __('File "config.php" could not be located/converted!','emergency-management');
      $mfem_result_message .= '<p>&nbsp;</p>';
    }
  } else {
    $feedback = 'prohibited';
  }
  
  add_option( 'mfem_admin_notice', $mfem_result_message, '', 'no' );
  
  nocache_headers();
  wp_safe_redirect( esc_url_raw( add_query_arg(
    array(
      'mfem_admin_notice' => $feedback,
    ),
    admin_url( 'admin.php?page=mfem-emergency-management&tab=keys-salts' ) 
  ) ) );
  
  status_header(200); // everything is OK
  exit();             // request-handler should always exit...
  
}


